@@ -1,8 +1,8 @@
 /*
    +----------------------------------------------------------------------+
-   | PHP Version 5                                                        |
+   | PHP Version 7                                                        |
    +----------------------------------------------------------------------+
-   | Copyright (c) 1997-2015 The PHP Group                                |
+   | Copyright (c) 1997-2018 The PHP Group                                |
    +----------------------------------------------------------------------+
    | This source file is subject to version 3.01 of the PHP license,      |
    | that is bundled with this package in the file LICENSE, and is        |
@@ -16,7 +16,7 @@
    +----------------------------------------------------------------------+
  */
 
-/* $Id: 7edea19a631e385746c56e87fbf770f82957b7fa $ */
+/* $Id: d8d6f46f29c667868a8d3630cd0d08003d0236bd $ */
 
 /* incorporated from D.J.Bernstein's cdb-0.75 (http://cr.yp.to/cdb.html)*/
 
@@ -43,18 +43,18 @@
 #endif
 
 /* {{{ cdb_match */
-static int cdb_match(struct cdb *c, char *key, unsigned int len, uint32 pos TSRMLS_DC)
+static int cdb_match(struct cdb *c, char *key, unsigned int len, uint32 pos)
 {
 	char buf[32];
 	unsigned int n;
 
 	while (len > 0) {
 		n = sizeof(buf);
-		if (n > len) 
+		if (n > len)
 			n = len;
-		if (cdb_read(c, buf, n, pos TSRMLS_CC) == -1) 
+		if (cdb_read(c, buf, n, pos) == -1)
 			return -1;
-		if (memcmp(buf, key, n)) 
+		if (memcmp(buf, key, n))
 			return 0;
 		pos += n;
 		key += n;
@@ -79,29 +79,29 @@ uint32 cdb_hash(char *buf, unsigned int len)
 /* }}} */
 
 /* {{{ cdb_free */
-void cdb_free(struct cdb *c TSRMLS_DC)
+void cdb_free(struct cdb *c)
 {
 }
 /* }}} */
 
 /* {{{ cdb_findstart */
-void cdb_findstart(struct cdb *c TSRMLS_DC)
+void cdb_findstart(struct cdb *c)
 {
 	c->loop = 0;
 }
 /* }}} */
 
 /* {{{ cdb_init */
-void cdb_init(struct cdb *c, php_stream *fp TSRMLS_DC)
+void cdb_init(struct cdb *c, php_stream *fp)
 {
-	cdb_free(c TSRMLS_CC);
-	cdb_findstart(c TSRMLS_CC);
+	cdb_free(c);
+	cdb_findstart(c);
 	c->fp = fp;
 }
 /* }}} */
 
 /* {{{ cdb_read */
-int cdb_read(struct cdb *c, char *buf, unsigned int len, uint32 pos TSRMLS_DC)
+int cdb_read(struct cdb *c, char *buf, unsigned int len, uint32 pos)
 {
 	if (php_stream_seek(c->fp, pos, SEEK_SET) == -1) {
 		errno = EPROTO;
@@ -112,7 +112,7 @@ int cdb_read(struct cdb *c, char *buf, unsigned int len, uint32 pos TSRMLS_DC)
 		do {
 			r = php_stream_read(c->fp, buf, len);
 		} while ((r == -1) && (errno == EINTR));
-		if (r == -1) 
+		if (r == -1)
 			return -1;
 		if (r == 0) {
 			errno = EPROTO;
@@ -126,7 +126,7 @@ int cdb_read(struct cdb *c, char *buf, unsigned int len, uint32 pos TSRMLS_DC)
 /* }}} */
 
 /* {{{ cdb_findnext */
-int cdb_findnext(struct cdb *c, char *key, unsigned int len TSRMLS_DC)
+int cdb_findnext(struct cdb *c, char *key, unsigned int len)
 {
 	char buf[8];
 	uint32 pos;
@@ -134,10 +134,10 @@ int cdb_findnext(struct cdb *c, char *key, unsigned int len TSRMLS_DC)
 
 	if (!c->loop) {
 		u = cdb_hash(key, len);
-		if (cdb_read(c, buf, 8, (u << 3) & 2047 TSRMLS_CC) == -1) 
+		if (cdb_read(c, buf, 8, (u << 3) & 2047) == -1)
 			return -1;
 		uint32_unpack(buf + 4,&c->hslots);
-		if (!c->hslots) 
+		if (!c->hslots)
 			return 0;
 		uint32_unpack(buf, &c->hpos);
 		c->khash = u;
@@ -148,22 +148,22 @@ int cdb_findnext(struct cdb *c, char *key, unsigned int len TSRMLS_DC)
 	}
 
 	while (c->loop < c->hslots) {
-		if (cdb_read(c, buf, 8, c->kpos TSRMLS_CC) == -1) 
+		if (cdb_read(c, buf, 8, c->kpos) == -1)
 			return -1;
 		uint32_unpack(buf + 4, &pos);
-		if (!pos) 
+		if (!pos)
 			return 0;
 		c->loop += 1;
 		c->kpos += 8;
-		if (c->kpos == c->hpos + (c->hslots << 3)) 
+		if (c->kpos == c->hpos + (c->hslots << 3))
 			c->kpos = c->hpos;
 		uint32_unpack(buf, &u);
 		if (u == c->khash) {
-			if (cdb_read(c, buf, 8, pos TSRMLS_CC) == -1) 
+			if (cdb_read(c, buf, 8, pos) == -1)
 				return -1;
 			uint32_unpack(buf, &u);
 			if (u == len)
-			switch(cdb_match(c, key, len, pos + 8 TSRMLS_CC)) {
+			switch(cdb_match(c, key, len, pos + 8)) {
 			case -1:
 				return -1;
 			case 1:
@@ -179,16 +179,16 @@ int cdb_findnext(struct cdb *c, char *key, unsigned int len TSRMLS_DC)
 /* }}} */
 
 /* {{{ cdb_find */
-int cdb_find(struct cdb *c, char *key, unsigned int len TSRMLS_DC)
+int cdb_find(struct cdb *c, char *key, unsigned int len)
 {
-	cdb_findstart(c TSRMLS_CC);
-	return cdb_findnext(c, key, len TSRMLS_CC);
+	cdb_findstart(c);
+	return cdb_findnext(c, key, len);
 }
 /* }}} */
 
 /* {{{ cdb_version */
-char *cdb_version() 
+char *cdb_version()
 {
-	return "0.75, $Id: 7edea19a631e385746c56e87fbf770f82957b7fa $";
+	return "0.75, $Id: d8d6f46f29c667868a8d3630cd0d08003d0236bd $";
 }
 /* }}} */
